package ru.dbotthepony.kstarbound.defs

import com.google.gson.GsonBuilder
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.io.json.KTypeAdapter
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import kotlin.properties.Delegates

class ParallaxPrototype {
	var verticalOrigin = 0.0

	var layers = Array(0) { ParallaxPrototypeLayer() }

	companion object {
		val ADAPTER = KTypeAdapter(::ParallaxPrototype,
			ParallaxPrototype::verticalOrigin,
			ParallaxPrototype::layers,
		)

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(ParallaxPrototype::class.java, ADAPTER)
			gsonBuilder.registerTypeAdapter(ParallaxPrototypeLayer::class.java, ParallaxPrototypeLayer.ADAPTER)
			gsonBuilder.registerTypeAdapter(ParallaxPrototypeLayer.Parallax::class.java, ParallaxPrototypeLayer.LAYER_PARALLAX_ADAPTER)
		}
	}
}

class ParallaxPrototypeLayer {
	class Parallax(val x: Double, val y: Double)

	var timeOfDayCorrelation: String? = null
	var offset = Vector2d.ZERO
	var repeatY = false
	var lightMapped = false
	var tileLimitTop: Int? = null
	var parallax by Delegates.notNull<Parallax>()

	var unlit = false
	var nohueshift = false
	var minSpeed = 0
	var maxSpeed = 0
	var fadePercent = 0.0

	var frequency = 1.0

	var modCount = 0

	var noRandomOffset = false
	var directives: String? = null

	var kind by Delegates.notNull<String>()
	var baseCount = 1

	companion object {
		val LAYER_PARALLAX_ADAPTER = object : TypeAdapter<Parallax>() {
			override fun write(out: JsonWriter?, value: Parallax?) {
				TODO("Not yet implemented")
			}

			override fun read(reader: JsonReader): Parallax {
				return when (val type = reader.peek()) {
					JsonToken.BEGIN_ARRAY -> {
						reader.beginArray()
						val instance = Parallax(reader.nextDouble(), reader.nextDouble())
						reader.endArray()
						instance
					}

					JsonToken.NUMBER -> {
						val num = reader.nextDouble()
						Parallax(num, num)
					}

					else -> throw JsonSyntaxException("Unexpected token $type")
				}
			}
		}

		val ADAPTER = KTypeAdapter(::ParallaxPrototypeLayer,
			ParallaxPrototypeLayer::timeOfDayCorrelation,
			ParallaxPrototypeLayer::offset,
			ParallaxPrototypeLayer::repeatY,
			ParallaxPrototypeLayer::lightMapped,
			ParallaxPrototypeLayer::tileLimitTop,
			ParallaxPrototypeLayer::parallax,
			ParallaxPrototypeLayer::unlit,
			ParallaxPrototypeLayer::nohueshift,
			ParallaxPrototypeLayer::minSpeed,
			ParallaxPrototypeLayer::maxSpeed,
			ParallaxPrototypeLayer::fadePercent,
			ParallaxPrototypeLayer::kind,
			ParallaxPrototypeLayer::baseCount,
			ParallaxPrototypeLayer::noRandomOffset,
			ParallaxPrototypeLayer::directives,
			ParallaxPrototypeLayer::frequency,
			ParallaxPrototypeLayer::modCount,
		)
	}
}
