package ru.dbotthepony.kstarbound.world.entities.projectile

import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.defs.projectile.ConfiguredProjectile
import ru.dbotthepony.kstarbound.world.World
import ru.dbotthepony.kstarbound.world.entities.Entity
import ru.dbotthepony.kstarbound.world.entities.IEntity
import ru.dbotthepony.kstarbound.world.entities.LogicalMovementController
import ru.dbotthepony.kstarbound.world.entities.MovementController

class Projectile(world: World<*, *>, val def: ConfiguredProjectile) : Entity(world) {
	override val movement: MovementController<Projectile> = (
		AbstractProjectileMovementController.factorize(this, def) ?:
		LogicalMovementController(this).also { LOGGER.error("No physics controller for ${def.physics}, defaulting to dummy movement controller!") }) as MovementController<Projectile>

	private var timeToLive = def.timeToLive
	private var markForDeath = false

	override fun thinkAI(delta: Double) {
		timeToLive -= delta

		if (timeToLive <= 0.0 || markForDeath) {
			detonate()
		}
	}

	fun markForDetonation() {
		markForDeath = true
	}

	fun collideWithEntity(other: IEntity) {
		// Can't do anything if we are technically dead
		if (markForDeath)
			return

		if (!def.piercing) {
			markForDeath = true
		}

		if (def.damageKind != null)
			other.dealDamage(def.power, def.damageKind, def.damageType)
	}

	fun detonate() {
		for (action in def.actionOnReap) {
			action.execute(this)
		}

		remove()
	}

	companion object {
		private val LOGGER = LogManager.getLogger(Projectile::class.java)
	}
}
